{*************************************************************************
 *                                                                       *
 *                    XI Olimpiada Informatyczna                         *
 *                                                                       *
 *   Rozwizanie zadania: Przeprawa                                      *
 *   Plik:                prz.pas                                        *
 *   Autor:               Pawe Wolff                                    *
 *   Data:                2003.10.29                                     * 
 *************************************************************************}

program Przeprawa;

const
  MAX_N = 16;
  MAX_T = 50;
  DWA_DO_MAX_N = 65536;


type
  TBajtolaz =
    record
      t: integer;
      w: integer;
    end;
    
  TTabBajtolazow = array [1..MAX_N] of TBajtolaz;
  
  TTabZnacznikow = array [1..MAX_N] of boolean;
  

var
  n: integer; { liczba Bajtoazw }
  b: TTabBajtolazow; { opis Bajtoazw }
  
  w: integer; { maksymalne dopuszczalne obcienie mostu }
  
  obliczonyCzas: array [0..DWA_DO_MAX_N - 1] of integer;
    { tu zapamitujemy obliczone ju czasy przeprawy podzbiorw zbioru
      wszystkich Bajtoazw }
  

procedure WczytajDane(var z: TTabZnacznikow);
var
  i: integer;
begin
  Readln(w, n);
  for i := 1 to n do begin
    with b[i] do
      Readln(t, w);
    z[i] := true;
  end;
end;


function NajwolniejszyBajtolaz(var z: TTabZnacznikow): integer;
{ zwraca indeks najwolniejszego nieprzeprawionego Bajtoaza; jeeli takiego
  nie ma, to zwraca 0 }
var
  i: integer;
  maxT: integer;  
begin
  maxT := 0;
  NajwolniejszyBajtolaz := 0;
  
  for i := 1 to n do
    if z[i] and (b[i].t > maxT) then begin
      NajwolniejszyBajtolaz := i;
      maxT := b[i].t;
    end;
end;


function Indeks(var z: TTabZnacznikow): longint;
{ przeksztaca tablic znacznikw z na indeks w tablicy obliczonyCzas }
var
  ind: longint;
  i: integer;

begin
  ind := 0;
  for i := 1 to n do begin
    ind := 2 * ind;
    if z[i] then
      Inc(ind);
  end;

  Indeks := ind;
end;

function ZapamietanyWynik(var z: TTabZnacznikow): integer;
{ zwraca zapamitany czas przeprawy podzbioru Bajtoazw okrelony w tablicy z }
begin
  ZapamietanyWynik := obliczonyCzas[Indeks(z)];
end;

procedure ZapamietajWynik(var z: TTabZnacznikow; wynik: integer);
{ zapisuje (w tablicy obliczonyCzas) minimalny czas (okrelony w zmiennej
  wynik) przeprawy podzbioru Bajtoazw okrelonego w tablicy z }
begin
  obliczonyCzas[Indeks(z)] := wynik;
end;


function ObliczWynik(var z: TTabZnacznikow): integer;
{ zwraca minimalny czas przeprawy podzbioru Bajtoazw okrelonego w tablicy z }

  procedure WydzielGrupe(var z: TTabZnacznikow; var wynik, wagaGrupy: integer;
                         ostatniWGrupie: integer);
  { powiksza grup Bajtoazw (zainicjowan w funkcji ObliczWynik)
    o Bajtoazw z podzbioru okrelonego przez tablic z i o indeksach
    wikszych ni ostatniWGrupie; robi to na wszystkie takie sposoby,
    ktre prowadz do tego, e wynikowa grupa (o inicjalnej sumarycznej wadze
    wagaGrupy) jest moliwie jak najwiksza (cile rzecz biorc - maksymalna
    w sensie inkluzji) oraz e jej waga nie przekracza w (dopuszczalnego
    obcienia mostu);
    dla wszystkich tak powikszonych grupy oblicza minimalny czas takiej
    przeprawy Bajtoazw (z podzbioru okrelonego przez tablic z), e
    najpierw przeprawiani s Bajtoazi z grupy, a nastpnie pozostali;
    przez parametr wynik przekazuje najkrtszy czas przeprawy wykonanej
    wg powyszego schematu }
  var
    i, wynikPom: integer;
    
  begin
    i := ostatniWGrupie + 1;
    while i <= n do begin

      { szukamy nastpnego Bajtoaza w podzbiorze okrelonym przez z }
      while (i <= n) and not z[i] do
        Inc(i);
      
      if i <= n then begin { jeli jeszcze kogo (i-tego Bajtoaza) mona doda
                             do grupy... }
        if wagaGrupy + b[i].w <= w then begin { i na dodatek powikszona
	                     o niego grupa nie przecia mostu, to... }
          { dodajemy i-tego Bajtoaza do przeprawianej grupy }
          z[i] := false;
          Inc(wagaGrupy, b[i].w);
	  
	  WydzielGrupe(z, wynik, wagaGrupy, i);

          { usuwamy i-tego Bajtoaza z przeprawianej grupy }
	  z[i] := true;
          Dec(wagaGrupy, b[i].w);
        end;
        Inc(i); { rozpatrujemy nastpnego Bajtoaza }
      end
    end;

    { sprawdzamy, czy grupa jest maksymalna... }
    i := 1;
    while (i <= n) and not (z[i] and (wagaGrupy + b[i].w <= w)) do
      Inc(i);

    if i > n then begin
      { grupa jest maksymalna... }
      wynikPom := ObliczWynik(z); { wic liczymy minimalny czas przeprawy
                                    reszty Bajtoazw }
      ZapamietajWynik(z, wynikPom);
      if wynikPom < wynik then
        wynik := wynikPom;
    end;
  end;

var
  i, wynik, wagaGrupy: integer;

begin
  wynik := ZapamietanyWynik(z); { moe ju dla takiego podzbioru Bajtoazw
                                  mamy policzony wynik? }
  if wynik <> 0 then { tak, ju go policzylimy }
    ObliczWynik := wynik
  else begin
    { nie, teraz bdziemy go liczy }
    i := NajwolniejszyBajtolaz(z);
    if i = 0 then begin { wszyscy s przeprawieni (z definiuje zbir pusty) }
      ZapamietajWynik(z, 0);
      ObliczWynik := 0
    end
    else begin
      wynik := MAX_N * MAX_T; { duej ju przeprawa trwa nie moe }

      wagaGrupy := b[i].w;
      z[i] := false;
      { inicjalnie grupa skada si z najwolniejszego Bajtoaza, ktry
        nie zosta jeszcze przeprawiony }
      WydzielGrupe(z, wynik, wagaGrupy, 0);
      z[i] := true;

      ObliczWynik := b[i].t + wynik;
    end;
  end;
end;



var
  doPrzeprawienia: TTabZnacznikow; { okrela, ktry z Bajtoazw jest jeszcze
                                     nieprzeprawiony }
  i: longint;
  dwaDoN: longint;

begin
  WczytajDane(doPrzeprawienia);

  { zerujemy tablice obliczonyWynik }
  dwaDoN := 1;
  for i := 1 to n do
    dwaDoN := 2 * dwaDoN;
  for i := 0 to dwaDoN - 1 do
    obliczonyCzas[i] := 0;

  { obliczamy i zapisujemy wynik }
  Writeln(ObliczWynik(doPrzeprawienia));
end.
